#!/bin/sh
# $Id: t-tlscnf-0.sh,v 1.9 2013/12/08 02:28:35 ca Exp $
# Copyright (c) 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.

# ---------------------------
# run t-tlscnf-0 with some configuration files
# ---------------------------

test -s stop && exit 1              

PRG=t-tlscnf-0
IN=${PRG}.in
OUT=${PRG}.out
OK=${PRG}.ok
ERR=${PRG}.err

./${PRG} -v > ${OUT} 2>${ERR}
if grep '^MTA_USE_TLS$' ${ERR} >/dev/null
then
  if grep '^MTA_TLS_DEBUG$' ${ERR} >/dev/null
  then
    TLSDBG="
  # debug options [experts only]
  debug=0;"
  fi
else
  echo "$0: SKIPPED"
  exit 0
fi

runtest()
{
./${PRG} ${IN} > ${OUT} 2>${ERR}
if diff ${OUT} ${OK}
then
  :
else
  echo "$0: FAIL: test ${TEST}"
  exit 1
fi
}

# ---------------------------
TEST=1
rm -f ${IN} ${OUT} ${OK}
cat > ${IN} <<EOF
tls {
  dhparam { file = /etc/meta1/mydh.param; }
}
EOF
cat > ${OK} <<EOF
tls {
  # DH parameters
  DHParam {
    # name of file with DH parameters
    file="/etc/meta1/mydh.param";
  }
  # size of TLS session cache
  cache_size=0;
  # timeout for TLS session cache
  cache_timeout=0s;
  # maximum checked depth of certificate chain for verification
  verify_depth=0;$TLSDBG
}
EOF
runtest

# ---------------------------
TEST=2
rm -f ${IN} ${OUT} ${OK}
cat > ${IN} <<EOF
tls {
  dhparam { bits = 1024; }
}
EOF
cat > ${OK} <<EOF
tls {
  # DH parameters
  DHParam {
    # bit length of predefined DH parameter
    bits=1024;
  }
  # size of TLS session cache
  cache_size=0;
  # timeout for TLS session cache
  cache_timeout=0s;
  # maximum checked depth of certificate chain for verification
  verify_depth=0;$TLSDBG
}
EOF
runtest

rm -f ${IN} ${OUT} ${OK} ${ERR}
exit 0
