#!/bin/sh
# $Id: t-delay-0.sh,v 1.23 2011/09/11 05:21:20 ca Exp $
# Copyright (c) 2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test delay message; including text of bounces delivered via lmtp to smtps3
# run smtp1.sh with different parameters
# ----------------------------------------
#

if test X"${MTA_NO_SLOW_TEST}" != "X"
then
  echo "SKIPPED: $0: MTA_NO_SLOW_TEST is set"
  exit 0
fi

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto qmgr MTA_DELAYED_DSN

OK="t-delay-0.ok"
OUT="t-delay-0.out"
TMP="t-delay-0.tmp"
EDBROK="t-delay-0.edbr"

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to working directory for alias file!
cd ../qmgr || exit 1

# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
user:	local:
sender: local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}
rm -f ${MTMAP}

# run some test programs

# ----------------------------------------
# temporary failure from SMTP sink; create a warning DSN; check content
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
${CAT} > ${EDBROK} <<EOF
got transaction
	mail=<sender@local.dom>
	rcpts_tot=2
	rcpts_left=1
	rcpts_temp=1
	rcpts_perm=0
	state=0
	aqt_rcpts_ar=0
got recipient
	rcpt=<451@local.host>
	aqr_rcpt_idx=0
	aqr_da_idx=0
	aqr_status=451
EOF
if ${SHELL} ${SD}/smtp1.sh -O ${EDBROK} -3 lmtpsock -Ssender@local.dom -R451@local.host -Q '-F 0 -M 20 -T w=5' -i 8 -s 30 -E '2|3' -X 0 -A 1
then
  if test -s ${B1}
  then
     ${EGREP} -v '^Date: ' ${B1} |\
	 ${SED} -e '/^Received:/,$d' \
		-e 's/
$//' -e 's/Mailer-Daemon@.*/Mailer-Daemon@/' > ${OUT}

     ${CAT} > ${OK} <<EOF
Return-Path: <>
From: Mailer-Daemon@
Subject: Delayed mail
To: <sender@local.dom>

${DSN_DEL}

Recipient:
<451@local.host>
Remote-MTA: 127.0.0.1
Reason:
451 whatever
during RCPT


Your original mail follows:
EOF
    if ${DIFF} -w ${OK} ${OUT} >/dev/null
    then
      ls ?/S0* > ${TMP}
      if test -s ${TMP}
      then
        # maybe check defedb: must have an entry?
        # check (x)delay: delay > xdelay
        L=`grep 'q_upd_rcpt_ok.* xdelay=' ${QML}`
        XD=`getle xdelay "$L"`
        D=`getle delay "$L"`
        if [ X"$XD" = "X" -o X"$D" = "X" ]
        then
          ${VERBOSE} && echo "$0: missing [x]delay in $L"
          ERR=true
        else
          if [ $XD -ge $D ]
          then
            ${VERBOSE} && echo "$0: xdelay=$XD is greater than delay=$D"
            ERR=true
          fi
        fi
      else
        ${VERBOSE} && echo "$0: missing CDB entry"
        ERR=true
      fi
    else
      ERR=true
    fi

  else
     ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi


# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  rm -f ${EDBROK} ${OK} ${OUT} ${TMP}
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
