#!/bin/sh
# $Id: t-tls-reqr-10.sh,v 1.1 2014/03/06 02:33:47 ca Exp $
#
# Copyright (c) 2014 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.
#
# ----------------------------------------
# test TLS requirements in smtpc; use chain of CAs; different digest
# creates two MTA (one in chkmts, one in qmgr) to use STARTTLS
# run smtpcnf.sh with different parameters
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

tlsrintro "$@"
setuptlsr0 "" "tls { digest=sha512;}" \
 "smtpc_rcpt_conf:@local.dom	tls_requirements { flags={verified}; cert_fps = { 36:16:73:A1:91:94:80:70:05:3F:A1:E0:E2:0D:76:01:93:0F:1B:FB:CE:8F:82:EA:CD:6C:2F:1E:7B:C1:D5:56:72:34:C5:04:16:0A:A6:B8:F6:39:15:AD:A9:87:B4:41:38:EF:5B:0B:85:CF:15:7F:50:60:90:9A:DF:D2:80:E4 }; }" \
 "tls { cert_file=cert4.pem; key_file=key4.pem; CAcert_file = ${CAS}; }"

# ----------------------------------------
# test: send mail to sink, requirements fulfilled
TEST=1
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -3 lmtpsock -Ssender${TEST}@other.dom -Ruser@local.dom -E 0 -X 0 -A 1
then
  chkTLS ${B1}
  cd ${THISDIR}
  egrep 'ERROR|FATAL|ALERT' ${SCL1} && ERR=true
  cd ../qmgr
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
rm -f ${B1}
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: send mail to sink, requirements not fulfilled
TEST=2
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

cd ${THISDIR}
${CAT} > ${ACC} <<EOF
smtpc_rcpt_conf:@local.dom	tls_requirements { flags={verified}; cert_fps = { 36:16:73:A1:91:94:80:70:05:3F:A1:E0:E2:0D:76:01:93:0F:1B:FB:CE:8F:82:EA:CD:6C:2F:1E:7B:C1:D5:56:72:34:C5:04:16:0A:A6:B8:F6:39:15:AD:A9:87:B4:41:38:EF:5B:0B:85:CF:15:7F:50:60:90:9A:DF:D2:80:E5 } }
EOF
rm -f ${ACCMAP}
${MM} -t'	' -F ${ACCMAP} < ${ACC}
# notify smar
kill -USR1 ${SMARPID}
cd ../qmgr

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -3 lmtpsock -Ssender${TEST}@other.dom -Ruser@local.dom -E 0 -X 0 -A 1
then
  if test -s ${B1}
  then
    ${VERBOSE} && echo "$0: test ${TEST}: found ${B1}"
    ERR=true
  else
    # check that mail is in queue, check error message...
    # how?
    cd ${THISDIR} || ERR=true
    ls ?/S* >/dev/null || ERR=true
    egrep 'ERROR|FATAL|ALERT' ${SCL1} && ERR=true
    cd ../qmgr
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
rm -f ${B1}
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


testend rmb
