#!/bin/sh
# $Id: t-tls-reuse-3.sh,v 1.5 2014/01/02 02:08:08 ca Exp $
#
# Copyright (c) 2011 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.
#
# ----------------------------------------
# test TLS session reuse: timeout (server)
# creates two MTA (one in chkmts, one in qmgr) to use STARTTLS
# run smtpcnf.sh with different parameters
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

tlsrintro "$@"
op_no_ticket
cd ${THISDIR} || exit 1
setuptls1

# ----------------------------------------
# start first MTA (chkmts, it will send the e-mails)
#
${CAT} > ${CONF2} <<EOF
smtpc { Log_Level = 16; remote_port=${SNKPORT}; wait_for_server=4;
	tls { cache_size = 8; cache_timeout=60;
		options = { SSL_OP_ALL, ${TICKET} };
	}
	flags = talk_to_myself; # don't check "talking to myself"
	start_action = wait; user = ${USER};
	path = "../smtpc/smtpc"; arguments = "smtpc -f ${CONF2}";
}
smtps {
	Log_Level = 12;
	daemon_address = localhost:${SRVPORT};
	wait_for_server=4;
	listen_socket { type=inet; port=${SRVPORT}; }
	start_action = wait; user = ${USER};
	path = ../smtps/smtps;
	arguments = "smtps -f ${CONF2} ${MTA_SERVER_OPTIONS}";
}
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
	smtpc { initial_connections=1; connections_max =2;}
	start_action = wait; user = ${USER};
	restart_dependencies = { smtps,smtpc,smar, };
	path = "../qmgr/qmgr"; arguments = "qmgr -f ${CONF2}";
}
smar { Log_Level = 12;
	start_action = wait; user = ${USER};
	restart_dependencies = { smtps,qmgr };
	path = "../smar/smar"; arguments = "smar -f ${CONF2}";
}
EOF

# version of output files
V=1
# path to programs
P=..
rm -f ${PID2}
trap killittls EXIT 2 15
# go for it
${P}/smar/smar -f ${CONF2} > a${V}.log 2>&1 &
echo $! >> ${PID2}
${P}/qmgr/qmgr -f ${CONF2} > q${V}.log 2>&1 &
echo $! >> ${PID2}
${P}/smtpc/smtpc -f ${CONF2} > c${V}.log 2>&1 &
echo $! >> ${PID2}
${P}/smtps/smtps -f ${CONF2} > s${V}.log 2>&1 &
echo $! >> ${PID2}

# check that the MTA is running
export P
${CHKD}/mta-running.sh ${SRVPORT} || killittls

export MTA_SNKPORT MTA_SRVPORT

# ----------------------------------------
# test: send mails to local address, check Received: header for TLS
# and check log for session reuse: should not be there because of timeout

TEST=1
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

cd ../qmgr
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT2}; wait_for_server=4;
  flags = talk_to_myself; # don't check "talking to myself"
}
smtps {
	Log_Level = 12;
	tls { cache_size = 8; cache_timeout=2; }
	daemon_address = localhost:${SRVPORT2};
	wait_for_server=4;
}
EOF

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -3 lmtpsock -Ssender@other.dom -Ruser@local.dom -E 0 -X 0 -A 2 -c "-m 2 -s 2 -W b=4" -Z "smtpc2"
then
  for f in ${B1} ${B2}
  do
    if test -s $f
    then
      if ${GREP} 'TLS' ${f} >/dev/null
      then
        :
      else
        ${VERBOSE} && echo "$0: test ${TEST}: TLS not found in ${f}"
        ERR=true
      fi
    else
      ${VERBOSE} && echo "$0: test ${TEST}: ${f} not found"
      ERR=true
    fi
  done

  for f in ${SSL} ../chkmts/c${V}.log
  do
    if grep 'starttls=successful, .*, se_reused=true' ${f} >/dev/null
    then
      ${VERBOSE} && echo "$0: test ${TEST}: se_reused=true not found in ${f}"
      ERR=true
    fi
  done

else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   # test X"${MTA_STOPONERROR}" != X && exit 1
fi
#rm -f ${B1} ${B2}
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
cd ${THISDIR}
rm -f ${MTMAP} ${ALIMAP}

testend
