/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: sm-conf-type-expr.c,v 1.21 2012/11/05 15:24:57 ca Exp $")

#if SM_LIBCONF_ALONE
#include <limits.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include "sm-conf.h"
#else /* SM_LIBCONF_ALONE */
#include "sm/limits.h"
#include "sm/error.h"
#include "sm/ctype.h"
#include "sm/memops.h"
#include "sm/string.h"
#include "sm/sm-conf.h"
#endif /* SM_LIBCONF_ALONE */

#include "sm-conf-node.h"
#include "sm-conf-state.h"
#include "sm-conf-type.h"
#include "sm-conf-util.h"
#include "sm-conf-u32.h"
#include "sm/sm-conf-expr.h"
#include "sm-expr.h"

static int
expr_write(
	sm_conf_T	*smc,
	sm_conf_node_T const		*node,
	sm_conf_definition_T const	*def,
	void		*data,
	cexpr_P		val)
{
	SM_IS_CONF_DEF(def);

	if (data != NULL)
		*(cexpr_P *)data = val;
	return 0;
}

#if 0
/*
**	Parameters:
**		smc -- context
**		node -- NULL or node, for error messages
**		def -- definition of the value
**		text -- text to scan
**		text_n -- # of bytes we can access
**		val_out -- result is stored here
**
**	Returns:
**		0 on sucecss, an error code on error.
*/

static int
sm_conf_expr_scan(
	sm_conf_T		*smc,
	sm_conf_node_T const	*node,
	sm_conf_definition_T const *def,
	char const		**text,
	size_t			*text_n,
	unsigned long	*val_out)
{
	char			loc[SM_CONF_ERROR_BUFFER_SIZE];
	int			err;
	size_t			i;
	unsigned long	val;
	char			*endptr;

	SM_IS_CONF_DEF(def);
	SM_REQUIRE(val_out != NULL);
	val = 0;
	*val_out = 0;

	err = sm_memtou32(*text, *text_n, &endptr, 0, &val);
	if (err != 0) {
		if (err == ERANGE)
			sm_conf_error_add(smc,
				"%s: overflow error in %s '%.*s'",
				sm_conf_node_location(smc, node,
					loc, sizeof loc),
				def->scd_name[0] == '\0' ?  "value" : def->scd_name,
				(int)*text_n, *text);
		else
			sm_conf_error_add(smc,
				"%s: expected number, got '%.*s'",
				sm_conf_node_location(smc, node, loc, sizeof loc),
				(int)*text_n, *text);
		return SM_CONF_ERR_TYPE;
	}

	i = endptr - *text;

	/* skip over leading white space */
	while (i < *text_n && ISSPACE((*text)[i]))
		i++;

	if (i < *text_n) {
		*text += i;
		*text_n -= i;
	}

	*val_out += val;
	return 0;
}
#endif /* 0 */

static int
sm_conf_type_expr_node_to_value(
	sm_conf_T			*smc,
	sm_conf_definition_T const	*def,
	uint32_t			 rqmnts,
	sm_conf_node_T		*node,
	void				*data)
{
	char const		*text;
	size_t			text_n;
	int				err;

	SM_IS_CONF_DEF(def);
	if (NULL == data)
		return 0;
	err = 0;

/* note:
something like { a + b -c; }
is not accepted: expected comma between list elements, got ';'
without ';' (which is a delimiter): { a + b -c }
sm_conf_node_to_value() fails: expected number, got list
maybe a different call helps?
strings (double-quotes) are not preserved:
{ 1 + 2 < var - "string" }
-> { 1 + 2 < var - string }
but this seems to work:
{ 1 + 2 < var - 'string' }
However, any of the special characters are not allowed,
and that includes '='! so we have another problem here.

it might be a better idea to change the parser
sm_conf_token()
to allow for "expressions" and split them into tokens.
how?
new type: SM_CONF_NODE_EXPR?
pass a flag: SM_CONF_TOKEN_FLAG_EXPR
how can we figure out that something is an expression?
tokenizing is completely(?) independent from parsing (is it?)
otherwise we could use context-dependent tokenizing

use
condition : expression
i.e., a different separator to let the tokenizer "know" what to do?
or start the expression with something "special", e.g., parentheses.

the problem with scanning (tokenizing) is that we may have more
than just numbers, variable, and strings: IPv{4,6} addresses,
hostnames, and even time specifications (6h 3m) that can contain
white space!
that makes at least using '.', ':' as separators impossible,
unless we treat those addresses/hostnames as strings (and hence
require quoting).
the tokenizer is a bit unusual, it allows basically every sequence
of characters that doesn't contain one of the ("basic") delimiters:

Punctuation allowed in identifiers:
  	! $ % & ' ( ) * + - . / : < > ?  @ [ \ ] ^ _ ` | ~
Punctuation NOT allowed in identifiers: { } , ; = # "

comments are only '#' to eol, not C/C++ comments, hence / is allowed.

if we allow more than just numbers, variable, and strings
then the lexer should "recognize" those types (or is it the parser?)
allowing white space in tokens makes things rather complicated.
this could be disallowed (e.g., time spec: 6h3m)

so: should we use just 3 basic types and all the rest is
handled via strings (quoted)?
What kind of functions/operations would we offer for IP addresses and times?
IP addresses: match address ranges (CIDR?)
times: usual comparisons (<, >, ...)

*/

	/* when can this happen? { } ? then we need a different lexer (mode)? */
	if (sm_conf_node_type(smc, node) == SM_CONF_NODE_LIST) {
		sm_conf_node_T	*sub;

		sub = NULL;
		while ((sub = sm_conf_list_next(smc, node, sub)) != NULL) {
			if (sm_conf_node_type(smc, sub) == SM_CONF_NODE_VALUE) {
			}
		}
	}
	else if (sm_conf_node_type(smc, node) == SM_CONF_NODE_VALUE) {
		lpctx_T lpctx;

		err = sm_conf_node_to_value(smc, "condition", node, &text, &text_n);
		if (err != 0) return err;

		/*
		 *  run yylex to count number of token so we can allocate an
		 *  expression tree?
		*/

		/* FIX ME! */
		err = m_expr_init(16, &lpctx.lpc_expr);
		if (sm_is_err(err))
			return err;

		lpctx.lpc_idtab = def->scd_idtab;
		lpctx.lpc_idsz = def->scd_idtabsz;
		lpctx.lpc_text = text;
		lpctx.lpc_text_n = text_n;
		lpctx.lpc_pos = 0;
		err = yyparse((void *)&lpctx);
		if (err != 0)
			return err;
		lpctx.lpc_expr->cex_rooti = lpctx.lpc_rooti;
		
		err = expr_write(smc, node, def, data, lpctx.lpc_expr);
	}

	return err;
}

static int
sm_conf_type_expr_value_check(
	sm_conf_T			*smc,
	sm_conf_definition_T const	*def,
	void const			*data)
{
	SM_IS_CONF_DEF(def);

	/* if we have a check function, use it. */
	if (def->scd_check != NULL)
		return (* def->scd_check)(smc, def->scd_check_data, def, data);
	return 0;
}

static int
sm_conf_type_expr_value_null(
	sm_conf_T			*smc,
	sm_conf_definition_T const	*def,
	void				*data)
{
	if (data != NULL) {
		SM_IS_CONF_DEF(def);

#if 0
/*
 * this is the wrong way to scan the default?
 */
		/* is there a default? if NULL: don't set value to 0 */
		if (def->scd_default != NULL) {
			char const		*text;
			size_t			text_n;
			unsigned long	val;
			int				err;

			text = def->scd_default;
			text_n = strlen(def->scd_default);
			err = sm_conf_expr_scan(smc, NULL, def, &text, &text_n, &val);
			if (err != 0) return err;

			err = expr_write(smc, NULL, def, data, val);
			if (err != 0) return err;
		}
#endif
	}
	return 0;
}

sm_conf_type_T const
sm_conf_type_expr_data =
{
	sm_conf_type_expr_node_to_value,
	sm_conf_type_expr_value_check,
	sm_conf_type_expr_value_null
};
