#!/bin/sh
# $Id: t-conf-sock-1.sh,v 1.4 2013/11/19 04:59:17 ca Exp $

# ------------------------------------------------------------
# test multiple socket specifications; uses t-conf-sock-1
# ------------------------------------------------------------

test -s stop && exit 1

ERRS=0

CNF=sms0.conf
OUT=sms0.out
OK=sms0.ok

PRG=./t-conf-sock-1

if $PRG -V 2>&1 | fgrep 'NETINET6' > /dev/null
then
  HAS_IPV6=true
else
  HAS_IPV6=false
fi

cat > ${CNF} <<EOF
listen_socket { type=unix; path="/a/b/path.sock";}
listen_socket { type=inet; port=23; address=1.2.3.4;}
EOF

cat > ${OK} <<EOF
path="/a/b/path.sock"
IPv4
port=23
address= 1.2.3.4
EOF

if ${HAS_IPV6}
then
  echo 'listen_socket { type=inet6; port=623; address=::1;}' >> ${CNF}
cat >> ${OK} <<EOF
IPv6
port=623
address= ::1
EOF
fi

# ----------------------------------------
TEST=1
if ${PRG} ${CNF} >${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    echo "$0: ${TEST} failed"
    ERRS=`expr ${ERRS} + 1 `
  fi
else
  ERRS=`expr ${ERRS} + 1 `
fi

# ----------------------------------------
TEST=2

mv ${OK} ${OUT}
cat > ${OK} <<EOF
# sockets to listen() on
listen_socket {
  # socket type
  type=unix;
  # pathname of Unix domain socket
  path="/a/b/path.sock";
  # umask to set before bind(2) and to set socket permissions
  umask=0;
}
# sockets to listen() on
listen_socket {
  # socket type
  type=inet;
  # port to listen on
  port=23;
  # bind to this IPv4 address
  address=1.2.3.4;
}
EOF
if ${HAS_IPV6}
then
cat >> ${OK} <<EOF
# sockets to listen() on
listen_socket {
  # socket type
  type=inet6;
  # port to listen on
  port=623;
  # bind to this IPv6 address
  address=0:0:0:0:0:0:0:1;
}
EOF
fi
cat ${OUT} >> ${OK}

if ${PRG} -s ${CNF} >${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    echo "$0: ${TEST} failed"
    ERRS=`expr ${ERRS} + 1 `
  fi
else
  ERRS=`expr ${ERRS} + 1 `
fi

# ----------------
if test ${ERRS} -eq 0
then
  rm -f ${CNF} ${OUT} ${OK}
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
