#!/bin/sh
# $Id: t-conf-0.sh,v 1.5 2011/11/10 20:42:57 ca Exp $
# Copyright (c) 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test dkim configuration file
# ----------------------------------------

test -s stop && exit 1              

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../libdkim
fi
ERRS=0
PRG=./t-dkim-conf-0
BODY=msg
CONF=t-conf-0.conf
OUT=t-conf-0.out

if $PRG -C | grep MTA_USE_DKIM >/dev/null
then
  :
else
   echo "SKIPPED: $0: $PRG not compiled with MTA_USE_DKIM"
   exit 0
fi

VERBOSE=false
if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

H=`pwd`
cd $SD/../libdkim || exit
K=`pwd`
cd $H || exit

# ----------------------------------------
# invalid configuration
TEST=1
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
cat >$CONF <<EOF
total_header_len = 2GB;
sign {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
}
EOF
# XREF: EDOM, error code in confchk.c
$PRG -E 33 -f $CONF -m $CONF -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# invalid configuration
TEST=2
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
cat >$CONF <<EOF
header_val_len_max = 1MB 1B;
sign {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
}
EOF
# XREF: EDOM, error code in confchk.c
$PRG -E 33 -f $CONF -m $CONF -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# invalid configuration: length to small
# XREF: DKIM_HDRLEN
TEST=3
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
cat >$CONF <<EOF
header_val_len_max = 1B;
sign {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
}
EOF
# XREF: EDOM, error code in confchk.c
$PRG -E 33 -f $CONF -m $CONF -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# invalid configuration: from= not unique
TEST=4
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
   from=example.com;
}
sign {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
   from=example.com;
}
sign net {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
}
EOF
# XREF: SM_E_KEYEXIST 
$PRG -I 0x8c04012c -f $CONF -m $CONF -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# invalid configuration: two selectors without from=
TEST=5
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
}
sign {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
   from=example.com;
}
sign net {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
}
EOF
# XREF: SM_CNF_E_INVALID
$PRG -I 0xc6040137 -f $CONF -m $CONF -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi



# ----------------------------------------
if test ${ERRS} -eq 0
then
  rm -f ${BODY} ${OUT} ${CONF}
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
