#!/bin/sh
# $Id: t-dkim-conf-4.sh,v 1.4 2013/12/13 02:31:41 ca Exp $
# Copyright (c) 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test dkim signing, using a configuration file
# ----------------------------------------

test -s stop && exit 1              

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../libdkim
fi
. ${SD}/common.sh

# ----------------------------------------
# sign based on From: address, using various from= pattern.
TEST=0
for F in example.com .com user@example.com user@.com user@
do
TEST=`expr $TEST + 1 `
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="hU1eiDdhQCwLZh1jXoUNF9Sg++8hJfXb4823xLTxNK2y6nMc6M6/iRudoNbF8/ijn/by/PnVE94Jphmzbr9OCfwh0zy+L9wd8ajH7ge7zPjQgUvuiLIzf33GtHYSrSGG4ry+LkUJNcKvjxI7UDSRlnodX6X3V6dWpto6l06yNeU"

cat >$BODY <<EOF
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
   from=${F};
}

sign net {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
}
EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed ${F}"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

done

# ----------------------------------------
if test ${ERRS} -eq 0
then
  rm -f ${BODY} ${OUT} ${CONF}
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
