/*
 * Copyright (c) 2003-2005 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: edbc.h,v 1.8 2011/02/28 02:30:42 ca Exp $
 */

#ifndef EDBC_H
#define EDBC_H 1

#include "sm/generic.h"
#include "sm/error.h"
#include "sm/magic.h"
#include "sm/time.h"
#include "sm/mta.h"
#include "sm/pthread.h"
#include "sm/bsd-tree.h"
#include "sm/queue.h"
#include "sm/edbc.h"

/*
**  Envelope database cache tree node.
**  Contains link to list of nodes.
*/

struct edbc_tnode_S
{
	RB_ENTRY(edbc_tnode_S)	 ectn_entry;
	ecn_hd_T		 ectn_hd;	/* head of list */
};

/*
**  NOTE: do not change this to another queue type
**  Some algorithms (edbc_next()) require that TAILQ_END(ectn_hd) is NULL.
*/

#define ECNL_INIT(ectn_hd)	TAILQ_INIT(ectn_hd)
#define ECNL_FIRST(ectn_hd)	TAILQ_FIRST(ectn_hd)
#define ECNL_END(ectn_hd)	TAILQ_END(ectn_hd)
#define ECNL_EMPTY(ectn_hd)	TAILQ_EMPTY(ectn_hd)
#define ECNL_NEXT(ecn)		TAILQ_NEXT(ecn, ecn_link)
#define ECNL_PRE(ectn_hd, ecn)	TAILQ_INSERT_HEAD(ectn_hd, ecn, ecn_link)
#define ECNL_APP(ectn_hd, ecn)	TAILQ_INSERT_TAIL(ectn_hd, ecn, ecn_link)
#define ECNL_REMOVE(ectn_hd, ecn) TAILQ_REMOVE(ectn_hd, ecn, ecn_link)

RB_PROTOTYPE(edbc_tree_S, edbc_tnode_S, ectn_entry, edbc_cmp)

sm_ret_T edbc_ht_init(edbc_ctx_P _edbc_ctx, uint _htsize);
sm_ret_T edbc_ht_add(edbc_ctx_P _edbc_ctx, edbc_node_P _edbc_node);
edbc_node_P edbc_ht_find(edbc_ctx_P _edbc_ctx, rcpt_id_T _rcpt_id);
sm_ret_T edbc_ht_rm(edbc_ctx_P _edbc_ctx, rcpt_id_T _rcpt_id);
uint edbc_hash(const char *_key);

#if SM_HT_PERF
# define htincradd(edbc_ctx)	((edbc_ctx)->edbc_adds++)
# define htincrlu(edbc_ctx)	((edbc_ctx)->edbc_lookups++)
# define htincrdel(edbc_ctx)	((edbc_ctx)->edbc_deletes++)
# define HTINCRACOL(edbc_ctx, hd)	do { \
		if (!LIST_EMPTY(hd))	\
			((edbc_ctx)->edbc_acollisions++);	\
	} while (0)
# define htincrlcol(edbc_ctx)	((edbc_ctx)->edbc_lcollisions++)
# define HTINCRDCOL(edbc_ctx, edbc_node)	do {	\
	if (LIST_NEXT(edbc_node, ecn_htl) != LIST_END(edbc_ctx) || \
		(edbc_node) != LIST_FIRST(edbc_ctx))	\
			(edbc_ctx)->edbc_dcollisions++;			\
	} while (0)
# define htincrgrows(edbc_ctx)	((edbc_ctx)->edbc_grows++)
# define htinitchain(edbc_ctx)	(edbc_ctx)->edbc_chainlen = 0
# define HTINCRCHAIN(edbc_ctx)	do {	\
		if (++(edbc_ctx)->edbc_chainlen > (edbc_ctx)->edbc_chainmax)	\
			(edbc_ctx)->edbc_chainmax = (edbc_ctx)->edbc_chainlen;\
	} while (0)
# define HTCHAINLEN(edbc_ctx, hd, edbc_node) do {	\
		edbc_node_P h;		\
		(edbc_ctx)->edbc_chainlen = 0;	\
		h = edbc_node;			\
		while (h != LIST_END(hd)) {				\
			++(edbc_ctx)->edbc_chainlen;	\
			h = LIST_NEXT(h, ecn_htl);	\
		}				\
		if ((edbc_ctx)->edbc_chainlen > (edbc_ctx)->edbc_chainmax)	\
			(edbc_ctx)->edbc_chainmax = (edbc_ctx)->edbc_chainlen;	\
	} while (0)

#else /* SM_HT_PERF */
# define htincradd(edbc_ctx)
# define htincrlu(edbc_ctx)
# define htincrdel(edbc_ctx)
# define HTINCRACOL(edbc_ctx, hd)
# define htincrlcol(edbc_ctx)
# define HTINCRDCOL(edbc_ctx, edbc_node)
# define htincrgrows(edbc_ctx)
# define htinitchain(edbc_ctx)
# define HTINCRCHAIN(edbc_ctx)
# define HTCHAINLEN(edbc_ctx, hd, edbc_node)
#endif /* SM_HT_PERF */

#if EDBC_HT_DEBUG
sm_ret_T edbc_ht_chk(edbc_ctx_P _edbc_ctx);
#else
# define edbc_ht_chk(edbc_ctx)	SM_NOOP
#endif

#endif /* EDBC_H */
