/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: convbackslash.c,v 1.3 2011/01/30 21:49:06 ca Exp $")
#include "sm/limits.h"
#include "sm/assert.h"
#include "sm/ctype.h"

/*
**  CONVBACKSLASH -- convert backslash C sequences in a string
**
**	Parameters:
**		src -- source string
**		dst -- destination string (should be as long as src)
**
**	Returns:
**		false if there was an error
**
**	Notes: NO length checks are made!
*/

bool
convbackslash(const char *src, char *dst)
{
	char c;

	while ((c = *src++) != '\0') {
		if (c == '\\') {
			int i;

			switch ((c = *src++)) {
			  case 'a': c = '\007'; break;
			  case 'b': c = '\b'; break;
			  case 'f': c = '\f'; break;
			  case 'n': c = '\n'; break;
			  case 'r': c = '\r'; break;
			  case 't': c = '\t'; break;
			  case 'v': c = 0x0B; break;
			  case '0':
				c = 0;
				for (i = 0; i < 3; i++) {
					if (*src >= '0' && *src <= '7')
						c = c*8 + *src++ - '0';
					else
						break;
				}
				break;
			  case '\0': src--; c = '\\'; break;
			  case '\\': break;
			  default:
				*dst++ = '\\';
			}
		}
		*dst++ = c;
	}
	*dst = c;
	return true;
}
